#!/bin/bash

# no comment
function PrintUsage
{
	echo $ERROR
	echo "Usage: installer.sh <options>  , where option is one or more of following:"
	echo "  -c, --clean         : cleans intermediate files from build"
	echo "  -m, --multi         : multi archive built - archive files along with RPMs"
	echo "  -i, --international : builds international archive"
	echo "  -r, --rpm           : build RPM distribution file"
	echo "  -a, --all           : build all necessary archives"
 	echo "  -n, --no-build      : doesn't perform build process"
	echo "  -s, --silent        : if no errors - skip warning and output messages"
	echo "  -?, -h, --help      : prints this help message"
	echo ""
	echo "Default is english tar.gz archive."
	echo ""
	exit 0
}

function GetIndex
{
	case $1 in
 		livemath) RET=0;;
   		lmviewer) RET=1;;
 	esac
  
  	return $RET
}

# Makes an install.sh script for specified binary
function MakeInstall
{
	directory=$1
	binary=$2

  	thefile=$directory/install.sh
  	
   	echo "#!/bin/bash" > $thefile
    	echo "" >> $thefile
   	echo "BINARY=$binary" >> $thefile
    
    	cat blank_install.sh >> $thefile
  	chmod +x $thefile
    
   	thefile=$directory/uninstall.sh
  	
   	echo "#!/bin/bash" > $thefile
    	echo "" >> $thefile
   	echo "BINARY=$binary" >> $thefile
    
    	cat blank_uninstall.sh >> $thefile
  	chmod +x $thefile
		
}

# Prepares Plugin Install script into specified directory
function MakeInstallPlugin
{
	directory=$1
 	
  	thefile=$directory/plugin_install.sh
  	
   	echo "#!/bin/bash" > $thefile
    	echo "" >> $thefile
 	cat blank_plugin.sh >> $thefile
  	chmod +x $thefile
 	
  	thefile=$directory/plugin_uninstall.sh
  	
   	echo "#!/bin/bash" > $thefile
    	echo "" >> $thefile
 	cat blank_unplugin.sh >> $thefile
  	chmod +x $thefile
}



# start the build script
function PerformBuild
{
	language=$1
	prod=$2
	silent=$3
		
 	echo "STARTING BUILD PROCESS:" 
	echo ""
	RESULT=0
	if [ "$language" == "multi" ]; then
		if [ "$silent" == "yes" ]; then
			./build.sh Release $prod all > /dev/null 2>.tmp.build.err # it should 'all' instead of TRS
			RESULT=$?
		else
			./build.sh Release $prod all # it should 'all' instead of TRS		
			RESULT=$?			
		fi 
		echo
	else
		if [ "$silent" == "yes" ]; then		
			./build.sh Release $prod > /dev/null 2>.tmp.build.err
			RESULT=$?
		else
			./build.sh Release $prod # the same as above
			RESULT=$?
		fi
 		echo
	fi

	if [ $RESULT != 0 ]; then
		echo "BUILD ERRROR !!!"
		if [ "$silent" == "yes" ]; then
			cat .tmp.build.err
			rm .tmp.build.err
		fi
		exit
	fi
	echo "BUILD READY !"
 
}

# prepare the directory needed for RPM build
# It is used for tar.gz archive as well
function PrepareDirTree
{
	dirroot=$1

 	pushd $dirroot > /dev/null

	# prepare new building area - to be deleted after that
	mkdir -p RPM
	mkdir -p RPM/RPMS
	mkdir -p RPM/RPMS/noarch
	mkdir -p RPM/BUILD

	# prepare new buildroot directory structure
	mkdir -p RPM/usr
	mkdir -p RPM/usr/local
	mkdir -p RPM/usr/local/lib
 	
 	popd > /dev/null
	
  	BUILDROOT="$dirroot/RPM/usr/local/lib"
	# set conf file for new building area
 	echo "%_topdir $dirroot/RPM" >> ~/.rpmmacros
}

function FinishDirTree
{
	dirroot=$1
 
 	pushd $dirroot > /dev/null
  	
  	rm -rf RPM > /dev/null
   
   	popd > /dev/null
}

# clear all CVS directories that may occur during copying.
function ClearCVS
{
	dirroot=$1
 
 	pushd $dirroot > /dev/null
  	for a in $( find ./ -type d -name "CVS" )
   	do
    	rm -rf $a
 	done
	popd > /dev/null
}


# prepare contents of one product depending on 
# language, binary, etc.
function PrepareContents
{
    #Delete any previous contents of xxx/RPM/usr/local
    # - rpmbuild is not happy to see there are files, which are not in .spec file
    rm -f -r $BINROOT/RPM/usr
	
    dirroot=$1
    binary=$2
    language=$3
    GetIndex $binary
    IDX=$?
    
    mkdir -p $dirroot/$binary
    dirroot=$dirroot/$binary
    echo "Preparing contents for $binary-$language"
   	

#vk: this is to make sure sh scripts in kdesetup will have proper permitions, because CVS may have lost 'X'
    chmod +x kdesetup/*.sh

    MakeInstall $dirroot $binary
    MakeInstallPlugin $dirroot
    
    eval pdirs=\$PRODUCTDIRS_$binary
  
    for adir in $pdirs
    do
     	mkdir -p $dirroot/$adir
    done
    
    eval pfiles=\$PRODUCTFILES_$binary

    src_file=""
    for afile in $pfiles
    do
    	if [ "$src_file" == "" ]; then
     		eval src_file=$afile
       		continue
     	fi
     
      	eval dst_file=$afile
	cp -f -p -r $src_file $dirroot/$dst_file > /dev/null
	if [ "$?" -ne "0" ]; then
  		echo Filed to copy one of the required files for distribution : $src_file 
		exit
	fi
	src_file=""
    done

    # fix the links stuff. Because of the IDIOTs that made Solaris !!!!
    # there is NO option for STOP DEREFERENCING links while copying. At least i didn't find it !
    pushd $dirroot > /dev/null
	rm curlang.uid
 	ln -s langs/en.uid curlang.uid    
    popd > /dev/null  
#   
    ClearCVS $dirroot
}

# Archive contents of prepared directory and 
# tarz them and gzips them
function MakeArchive
{
    archive=$1
  	binary=$2
  	addition=$3
   
   	directory="$BUILDROOT/$binary"
    
    GetIndex $binary
    
    IDX=$?
    ar_version=${VERSION[$IDX]}
    ar_build=${BUILD[$IDX]}
    
    platform=$PLATFORM
   	
    ar_name="$archive-$ar_version-$ar_build-$platform-$LANGUAGE.tar"
   	fullname="$BINROOT/$ar_name"
    
    echo "Creating archive: $ar_name"
    
	pushd $directory > /dev/null
 	
  	tar -cf $fullname *
	 
  	for addfile in $addition
  	do
   		tar -uf $fullname $addfile
   	done
 	popd > /dev/null
  
  	echo "Compresing..."
	gzip -f $fullname
}

# Prepare spec file needed for RPM creating
function PrepareRPMSpec
{
  	archive=$1
  	binary=$2
   	addition=$3
   
	specfile=$BINROOT/$binary.spec
 	GetIndex $binary
  
  	IDX=$?
 	product_name=${PRODUCTNAME[$IDX]}
  	description=${DESCRIPTION[$IDX]}
   	prodversion=${VERSION[$IDX]}
    prodbuild=${BUILD[$IDX]}
   	
    # prepare header part of SPEC file
	echo "Name: $archive-$LANGUAGE" > $specfile
	echo "Summary: $product_name" >> $specfile
	rpm_version=`rpm --version | grep "4.4."`
 	if [ "$rpm_version" == "" ]; then
		echo "Copyright: commercial" >> $specfile
	else
		#new rpmbuild says "Copyright" is depricted	
		echo "License: commercial" >> $specfile	
  	fi
 	
	echo "Group: Application/Internet" >> $specfile
	echo "URL: http://www.livemath.com" >> $specfile
	echo "Vendor: MathMonkeys, LLC" >> $specfile
	echo "Prefix: /usr/local/lib" >> $specfile
	echo "Version: $prodversion" >> $specfile
 	echo "Release: $prodbuild" >> $specfile
	echo "BuildRoot: $BINROOT/RPM" >> $specfile

 	#now prepare the files part
 	echo "%description" >> $specfile
	echo "$description" >> $specfile
	echo "" >> $specfile
	echo "%prep" >> $specfile
	echo "%build" >> $specfile
	echo "%install" >> $specfile
	echo "%files" >> $specfile
#put root 
	echo "%defattr(-,root,root)" >> $specfile 
 	echo "/usr/local/lib/$binary" >> $specfile

  	# add all additional files that are passed to the procedure
   	for addfile in $addition
    	do
   		echo "$addfile" >> $specfile
   	done
    	
    # add footer part of spec file
	echo "%post" >> $specfile
	echo "cd \$RPM_INSTALL_PREFIX" >> $specfile
 	echo "cd $binary" >> $specfile
	echo "./install.sh -q" >> $specfile
	echo "" >> $specfile
	echo "%preun" >> $specfile
	echo "cd \$RPM_INSTALL_PREFIX" >> $specfile
 	echo "cd $binary" >> $specfile
	echo "./uninstall.sh" >> $specfile
	echo "" >> $specfile
	echo "%postun" >> $specfile
	echo "cd \$RPM_INSTALL_PREFIX" >> $specfile
	echo "rm -rf $binary" >> $specfile

 	# the SPEC file is ready !!!
}

# Starts RPM build process and perform necessary
# copying/moving, etc.
function MakeRPM
{
 	archive=$1
  	binary=$2
   	addition=$3

    echo "Making $archive...rpm..."
   	PrepareRPMSpec $1 $2 $3

	# set conf file for new building area
 	echo "%_topdir $BINROOT/RPM" >> ~/.rpmmacros
        
    pushd $BINROOT > /dev/null
	
	target_option="--target noarch"
 	rpm_version=`rpm --version | grep "4.0.4"`
 	if [ "$rpm_version" == "" ]; then
  		target_option="--target=noarch"
  	fi
 	
   	specfile=$BINROOT/$binary.spec

  	rpm -bb $target_option $specfile > /dev/null 2>/dev/null
     	if [ "$?" -ne "0" ]; then
  	rpmbuild -bb $target_option $specfile > .tmp.rpm.out  2>.tmp.rpm.err
        if [ "$?" -ne "0" ]; then 
	cat .tmp.rpm.err 
	exit
	fi
	rm .tmp.rpm.out
	rm .tmp.rpm.err
    fi

	mv -f $BINROOT/RPM/RPMS/noarch/* ./
 
 	rm -f *.spec
	popd > /dev/null
}


##### BEGINING
# parse command line
LANGUAGE="enl"
CLEAN="no"
ERROR=""
BUILDRPM="no"
BUILDTAR="yes"
MAKEALL="no"
DOBUILD="yes"
SILENT="no"

for option in $@;
do
	case $option in
		-c | --clean) CLEAN="yes";;
		-m | --multi) BUILDRPM="yes" BUILDTAR="yes";;
  		-i | --international) LANGUAGE="multi";;
		-r | --rpm) BUILDRPM="yes" BUILDTAR="no";;
 		-a | --all) MAKEALL="yes" BUILDRPM="yes" BUILDTAR="yes" LANGUAGE="enl";;
   		-n | --no-build) DOBUILD="no";;
		-s | --silent) SILENT="yes";;
		-? | -h | --help) PrintUsage;;
		*) ERROR="wrong parameter";;
	esac
done

if [ "$ERROR" != "" ]; then
	echo $ERROR
	PrintUsage
fi

# include defaults to get binary root from there
source ./default.sh

# process language prior build, because the new resources should be build this way
if [ "$LANGUAGE" != "multi" ] && [ "$DOBUILD" == "yes" ] && [ -e $BINROOT ]; then
  	# clean those files, that are not needed for the archives and will obscure further english builds
    rm -rf $BINROOT/TRS/resgen > /dev/null
    rm -rf $BINROOT/TVS/resgen > /dev/null

    rm -f $BINROOT/TRS/livemath.uid > /dev/null
    rm -f $BINROOT/TVS/lmviewer.uid > /dev/null

    rm -f $BINROOT/TRS/langs/* > /dev/null
    rm -f $BINROOT/TVS/langs/* > /dev/null
fi

# first perform building !!!
if [ "$DOBUILD" == "yes" ]; then
	PerformBuild $LANGUAGE TRS $SILENT
	PerformBuild $LANGUAGE TVS $SILENT	
fi

#### HERE IS THE "SETUP" INFORMATION FOR INSTALLER.SH
# get binary root's absolute path as default directory
pushd . > /dev/null
cd $BINROOT
BINROOT=`pwd -P`
popd > /dev/null
SCRIPTS=`pwd -P`

# remember version, build number and platform for each product
PRODUCTNAME=("LiveMath Maker" "LiveMath Viewer")
DESCRIPTION=("Interactive algebra system" "Algebra visualization system")
VERSION=(`cat $BINROOT/TRS/version.info` `cat $BINROOT/TVS/version.info`)
BUILD=(`cat $BINROOT/TRS/buildid.info` `cat $BINROOT/TVS/buildid.info`)
PLATFORM=`cat $BINROOT/platform.info`


# the contents of each product
PRODUCTFILES_livemath='$BINROOT/TRS/curlang.uid ./ $BINROOT/TRS/livemath ./ $BINROOT/TRS/NewNote.the ./ $BINROOT/TRS/langs/* langs/ $SCRIPTS/setlang ./ $BINROOT/TRS/Starters/* Starters/ $SCRIPTS/kdesetup/* kdesetup/ $SCRIPTS/livemath.readme ./ $BINROOT/THS/nplivemath.so ./' # Don't forget to change blank_install.sh as well !
PRODUCTFILES_lmviewer='$BINROOT/TVS/curlang.uid ./ $BINROOT/TVS/lmviewer ./ $BINROOT/TVS/NewNote.the ./ $BINROOT/TVS/langs/* langs/ $SCRIPTS/setlang ./ $BINROOT/TVS/Starters/* Starters/ $BINROOT/THS/nplivemath.so ./'

# the additional subfolder that has to be created in each product directory
PRODUCTDIRS_livemath="langs Starters kdesetup"
PRODUCTDIRS_lmviewer="langs Starters"

### if necessary make cleaning
if [ "$CLEAN" == "yes" ]; then
	echo "Cleaning..."
	./clean.sh
	echo "Done with cleaning."
fi

# preare whole necessary tree
BUILDROOT=""
PrepareDirTree $BINROOT

# prepare contents of directory contatining LMM
#vk: moved below PrepareContents $BUILDROOT livemath $LANGUAGE

# prepare contents of directory contatining LMV
#vk:moved below PrepareContents $BUILDROOT lmviewer $LANGUAGE


if [ "$BUILDTAR" == "yes" ]; then
	echo "Building TAR.GZ archive(s)..."
	PrepareContents $BUILDROOT livemath $LANGUAGE 	
 	MakeArchive livemmaker livemath

	PrepareContents $BUILDROOT lmviewer $LANGUAGE
  	MakeArchive lmviewer lmviewer
 	echo "Done."
fi

if [ "$BUILDRPM" == "yes" ]; then
	echo "Building RPM archive(s)..."
# prepare contents of directory contatining LMM
	PrepareContents $BUILDROOT livemath $LANGUAGE
 	MakeRPM livemmaker livemath

	PrepareContents $BUILDROOT lmviewer $LANGUAGE
  	MakeRPM lmviewer lmviewer
	echo "Done."
fi

if [ "$MAKEALL" == "yes" ]; then
	add_args=""
 	if [ "$DOBUILD" == "no" ]; then
		add_args=' -n'
	fi
	silent_args="";
	if [ "$SILENT" == "yes" ]; then
		silent_args=' -s'		
	fi
	./installer.sh -i -m $add_args $silent_args
fi

FinishDirTree $BINROOT
 
